import socket
import time
from datetime import datetime
from ipaddress import IPv4Address, AddressValueError
from pprint import pprint


def data_processing(response, whois=''):
    if whois == "whois.iana.org":
        whois_d = ""
        for resp in response.splitlines():
            if resp.startswith('%') or not resp.strip():
                continue
            elif resp.startswith('whois'):
                whois_d = resp.split(":")[1].strip()
                break
        return whois_d if whois_d else False
    else:
        whois_ip = dict()
        num = 0
        for ln in response.splitlines():
            if ln.strip().startswith("%") or not ln.strip():
                continue
            else:
                if ln.strip().split(": ")[0].strip() in ['created', 'last-modified']:
                    dt = datetime.fromisoformat(ln.strip().split(": ")[1].strip()).strftime("%Y-%m-%d %H:%M:%S")
                    whois_ip.update({f'{ln.strip().split(": ")[0].strip()}_{num}': dt})
                    num += 1
                else:
                    whois_ip.update({ln.strip().split(": ")[0].strip(): ln.strip().split(": ")[1].strip()})
        return whois_ip if whois_ip else False


def get_whois(ip, whois):
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.connect((whois, 43))
    s.send((ip + "\r\n").encode())
    response = b""
    while True:
        data = s.recv(4096)
        response += data
        if not data:
            break
    s.close()
    data = data_processing(response.decode(), whois)
    return data


def validate_request(ip):
    try:
        IPv4Address(ip)
        if whois := get_whois(ip, 'whois.iana.org'):
            time.sleep(1)
            if info := get_whois(ip, whois):
                pprint(info)
            else:
                print("No IP address data has been received.")
        else:
            print("I can't get information about the registrar. The registrar whois.ripe.net will be used.")
            if info := get_whois(ip, 'whois.ripe.net'):
                pprint(info)
            else:
                print("No IP address data has been received.")
    except AddressValueError:
        print("IP-address not valid")
    except ConnectionResetError as ex:
        print(ex)


validate_request('82.98.86.175')
